
//%START program
import nsrunit; unit conversion on;
 math VascularOperator {

 // INDEPENDENT VARIABLE
 realDomain t sec; t.min=0; t.max=25; t.delta=0.1;
 // Flow parameters
 real F(t) L/min, Tinj=0.5 sec, InjRate=8 mg/sec,
      Cin(t) mg/L, V=250 ml, Fbar = 45 ml/sec;
 
 Cin = if(t<Tinj) InjRate/Fbar else 0;
 real K=0.5 dimensionless;
 real w sec^(-1);
 real n = 1 sec^(-1);
 real w = 2*PI * n;
 real phi = 0;
 F=Fbar*(1+K*sin(w*t+phi));
 // PARAMETERS
 real C0     = 0 mg/L,                 // initial concentration
      totalDelay = 5.55555 sec,            // total transit time required
      RD     = 0.28 dimensionless,    // Relative dispersion
      RD4th dimensionless,           // RD of a 4th order operator
      zeta1 = 0.95 dimensionless,    // 1st 2nd order op damping
      zeta2 = 0.80 dimensionless,    // 2nd 2nd order op damping
      wratio = 1.82 dimensionless,   // wratio = w2/w1
      Zeta1 dimensionless;           // protects bad values of zeta1
                                     
 //extern real Cin(t) mM;               // Input concentration
 real Cout(t) mg/L;                     // Output Concentration

 // CALCULATED PARAMETERS AND VARIABLES
 real rd dimensionless,            // Enforces 0<=RD<=RD4th
      pureDelay sec,               // the pure delay  transit time
      op4Delay sec,                // 4th order operator transit time
      w1(t) sec^(-1),                 // 1st 2nd order op frequency
      w2(t) sec^(-1),                 // 2nd 2nd order op frequency
      Cdelay(t) mg/L,                // output of pure delay operator
     
      C1(t) mg/L, dC1dt(t) mg/L/sec,   // 1st 2nd op variables
      C2(t) mg/L, dC2dt(t) mg/L/sec,   // 2nd 2nd op variables
      C3(t) mg/L, dC3dt(t) mg/L/sec,   // 3rd 2nd op variables
      C4(t) mg/L, dC4dt(t) mg/L/sec,   // 4th 2nd op variables
      C5(t) mg/L, dC5dt(t) mg/L/sec,   // 5th 2nd op variables
      C6(t) mg/L, dC6dt(t) mg/L/sec;   // 6th 2nd op variables

 // CALCULATIONS
 // Numeric Protection if z1 severely underdamped
 Zeta1= /* if(zeta1^2 < (wratio^2+1)/(2*wratio^2) )
        sqrt( (wratio^2+1)/(2*wratio^2) ) else  */ zeta1;
 
 RD4th = sqrt(3)*sqrt(zeta2^2-1/2 + wratio^2*(Zeta1^2-1/2) )/(zeta2 + Zeta1*wratio);  //Eq. 12

 rd = if (RD<0) 0                           // Enforce 0<=RD<=RD4th
      else if (RD>RD4th) RD4th 
      else RD;

 op4Delay = if(3*RD/RD4th<1) (3*RD/RD4th)*totalDelay else totalDelay;           // Calculate op4 transit time      Eq. 5     
 pureDelay = totalDelay-op4Delay;            // Calculate pure delay            Eq. 6

// w1 =if (op4Delay>0) (2/(op4Delay/3))*(Zeta1+zeta2/wratio) else 0;  
 w1=2*PI*(7.054317592)*F/V;              //Eq. 11
 w2 = w1*wratio;

 // INITIAL CONDITIONS
 when (t=t.min) {C1=0; dC1dt=0; C2=0; dC2dt=0;
                 C3=0; dC3dt=0; C4=0; dC4dt=0;
                 C5=0; dC5dt=0; C6=0; dC6dt=0; }

 // PURE DELAY AND ODE CALCULATIONS
 Cdelay = if(t<=t.min+pureDelay) C0  
          else Cin(t-t.min-pureDelay);       // Calculate delayed output

 C1:t = dC1dt;
 dC1dt:t = -2*Zeta1*w1*dC1dt +w1^2*(Cdelay-C1);  
 C2:t = dC2dt;
 dC2dt:t = -2*zeta2*w2*dC2dt + w2^2*(C1-C2);     

 C3:t = dC3dt;
 dC3dt:t = -2*Zeta1*w1*dC3dt +w1^2*(C2-C3);  
 C4:t = dC4dt;
 dC4dt:t = -2*zeta2*w2*dC4dt + w2^2*(C3-C4);     

 C5:t = dC5dt;
 dC5dt:t = -2*Zeta1*w1*dC5dt + w1^2*(C4-C5);  
 C6:t = dC6dt;
 dC6dt:t = -2*zeta2*w2*dC6dt + w2^2*(C5-C6);     

 // SELECT OUTPUT FROM EITHER DELAY OR 4th op
 Cout = if (op4Delay>0) C6 else Cdelay;

 // CALCULATE ALL STATISTICS FROM Cin and Cout
 real aI mg/L*sec, tI sec, RDI dimensionless,     // Cin  area, transit time, and RD
      skewI dimensionless, kurtI dimensionless, // Cin  skewness and kurtosis
      aO mg/L*sec, tO sec, RDO dimensionless,     // Cout area, transit time, and RD
      skewO dimensionless, kurtO dimensionless, // Cout skewness and kurtosis
      tsys sec, RDsys dimensionless;            // System transit time and RD
 curvesStatistics(Cin@t,Cout@t, aI,tI,RDI,skewI,kurtI,
                                aO,tO,RDO,skewO,kurtO,
                                tsys,RDsys);

//%END program
 }


 
